echo '----- START CONFIGURE SELENIUM -----';

# Conda
CondaArchive='https://repo.anaconda.com/miniconda/Miniconda3-py39_4.12.0-Linux-x86_64.sh';
CondaFile='/home/amnesia/miniconda3.sh';
CondaDirectory='/home/amnesia/miniconda3';
CondaActivate=''${CondaDirectory}'/bin/activate';
TBDriverScript=''${CondaDirectory}'/lib/python3.9/site-packages/tbselenium/tbdriver.py';

# Gecko
SeleniumDirectory='/home/amnesia/.selenium';
Profile='/home/amnesia/.tor-browser/profile.default';
TorPath='/usr/local/lib/tor-browser';
GeckoAddress='https://github.com/mozilla/geckodriver/releases/download/v0.31.0/geckodriver-v0.31.0-linux64.tar.gz';
GeckoSignAddress='https://github.com/mozilla/geckodriver/releases/download/v0.31.0/geckodriver-v0.31.0-linux64.tar.gz.asc';
GeckoArchive=''${SeleniumDirectory}'/geckodriver.tar.gz';
GeckoArchiveSign=''${SeleniumDirectory}'/geckodriver.tar.gz.asc';
GeckoPort=$( echo $((49152 + RANDOM % 16382)); );

# Key
MozillaPubkeyFile=''${SeleniumDirectory}'/mozilla_pubkey.asc';
MozillaFingerprint='14F26682D0916CDD81E37B6D61B7B526D98F0353';
MozillaPubkeyAddress='https://keys.openpgp.org/vks/v1/by-fingerprint/'${MozillaFingerprint}'';

# Download conda and tbselenium
wget -O ${CondaFile} ${CondaArchive};
[ $? -eq 0 ] || { echo "[ERROR] Can't download Conda. Exiting."; exit 1; };
echo '[INFO] Conda downloaded';

bash ${CondaFile};
[ $? -eq 0 ] || { echo "[ERROR] Can't install Conda. Exiting."; exit 1; };
echo '[INFO] Conda installed';
rm ${CondaFile};

source ${CondaActivate};
python3 -m pip --proxy socks5://127.0.0.1:9050 install tbselenium;
[ $? -eq 0 ] || { echo "[ERROR] Can't install Selenium. Exiting."; exit 1; };

# Fix tbselenium
sed -r -i 's/^(.*"--marionette-port", "2828"\]).*/\1, port = '${GeckoPort}'/' ${TBDriverScript};
echo '[INFO] tbselenium fixed';

# Create gecko dir
mkdir ${SeleniumDirectory};
echo '[INFO] Selenium Directory: '${SeleniumDirectory}'';

# Link browser
ln -s ${TorPath} ${SeleniumDirectory}/Browser;
echo '[INFO] Browser linked';

# Copy profile
cp -r ${Profile} ${SeleniumDirectory};
echo '[INFO] Profile copied';

# Download and verify gecko
wget -O ${MozillaPubkeyFile} ${MozillaPubkeyAddress};
[ $? -eq 0 ] || { echo "[ERROR] Can't download Mozilla Key. Exiting."; exit 1; };
gpg --import ${MozillaPubkeyFile};
gpg --batch --yes --sign-key ${MozillaFingerprint};
[ $? -eq 0 ] || { echo "[ERROR] Can't sign Mozilla Key. Exiting."; exit 1; };
rm ${MozillaPubkeyFile};
echo '[INFO] Mozilla Key imported';
wget -O ${GeckoArchive} ${GeckoAddress};
[ $? -eq 0 ] || { echo "[ERROR] Can't download Gecko Archive. Exiting."; exit 1; };
wget -O ${GeckoArchiveSign} ${GeckoSignAddress};
[ $? -eq 0 ] || { echo "[ERROR] Can't download Gecko Signature. Exiting."; exit 1; };
gpg --verify ${GeckoArchiveSign};
[ $? -eq 0 ] || { echo "[ERROR] Something's wrong with the signature!"; exit 1; };
echo '[INFO] Geckodriver downloaded and verified';

# Untar gecko
tar -x -f ${GeckoArchive} -C ${SeleniumDirectory};
echo '[INFO] Geckodriver uncompressed';
rm ${GeckoArchive};
rm ${GeckoArchiveSign};

# Open gecko port
sudo iptables -I OUTPUT -p tcp -s 127.0.0.1 -d 127.0.0.1 --dport ${GeckoPort} -j ACCEPT;
echo '[INFO] Ports opened';

echo '----- END CONFIGURE SELENIUM -----';
